@@ -73,6 +73,10 @@ enum
 
 #define TIMEOUT_TIME  1000 /* milliseconds */
 
+#define PROGRESS_TIME 5*60 /* seconds */
+
+#define UNUSED(x)     (void) x
+
 /* prototypes */
 static void
 smart_construct (XfcePanelPlugin *plugin);
@@ -165,7 +169,6 @@ smart_update_channels_watch_func(GPid pid, gint status, gpointer user_data)
   FILE *file;
   gchar *msg;
   gchar *s;
-  int c;
   
   packages = -1;
   
@@ -266,6 +269,7 @@ smart_update_channels_setup_child( gpointer   user_data)
 {
   SmartPlugin *smart = user_data;
   /* nothing to do */
+  UNUSED(smart);
 }
 
 static int
@@ -311,6 +315,7 @@ smart_update_channels( SmartPlugin    *smart)
   }
 
   smart->in_progress = TRUE;
+  g_get_current_time(&smart->in_progress_since);
 
   if (terminal)
   {
@@ -431,7 +436,6 @@ smart_check_upgrades_watch_func(GPid pid, gint status, gpointer user_data)
   FILE *file;
   gchar *msg;
   gchar *s;
-  int c;
   
   seen_header = FALSE;
   packages = 0;
@@ -477,10 +481,20 @@ smart_check_upgrades_watch_func(GPid pid, gint status, gpointer user_data)
   {
     while ( (p = fgets(buf, sizeof(buf), file)) != NULL )
     {
+      if (strstr(p, "Failed to write temporary StateFile") != NULL &&
+          smart->package_manager == PACKAGE_MANAGER_APT)
+      {
+        /* skip this error, we don't want to write anything anyway */
+        status = 0;
+        continue;
+      }
       if (strstr(p, "error: ") != NULL ||
           smart->package_manager != PACKAGE_MANAGER_SMART)
       {
-        p += strlen("error: ");
+        if (strstr(p, "error: ") == 0)
+          p += strlen("error: ");
+        if (strstr(p, "E: ") == 0)
+          p += strlen("E: ");
         msg = g_strdup(p);
         p = msg + strlen(msg) - 1;
         if (*p == '\n') *p = '\0';
@@ -506,8 +520,10 @@ smart_check_upgrades_watch_func(GPid pid, gint status, gpointer user_data)
     gtk_tooltips_set_tip(smart->tooltip, smart->button, s, NULL);
     if (smart->show_application)
       smart_set_icon(smart, smart->application_icon, smart->error_icon);
+    else if (smart->updates_available <= 0)
+      smart_set_icon(smart, smart->nopackage_icon, smart->error_icon);
     else
-      smart_set_icon(smart, smart->application_icon, smart->error_icon);
+      smart_set_icon(smart, smart->newpackage_icon, smart->error_icon);
     g_free(s);
   }
  
@@ -528,6 +544,7 @@ smart_check_upgrades_setup_child( gpointer   user_data)
 {
   SmartPlugin    *smart = user_data;
   /* nothing to do */
+  UNUSED(smart);
 }
 
 static int
@@ -577,6 +594,7 @@ smart_check_upgrades( SmartPlugin    *smart)
   }
 
   smart->in_progress = TRUE;
+  g_get_current_time(&smart->in_progress_since);
 
   if (!g_spawn_async_with_pipes( NULL /*cwd*/, argv, envp,
     G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_LEAVE_DESCRIPTORS_OPEN,
@@ -633,7 +651,6 @@ smart_sudo_smart_gui(SmartPlugin *smart, gchar *args)
   GError *error = NULL;
   gchar *program;
   gchar *command;
-  gboolean ok;
   
   switch (smart->package_manager)
   {
@@ -658,7 +675,7 @@ smart_sudo_smart_gui(SmartPlugin *smart, gchar *args)
         program = g_strdup(APT_PROGRAM); /* it is using consolehelper */
       break;
     default:
-      break;
+      return;
   }
 
   command = g_strdup_printf("%s%s%s%s", program,
@@ -716,7 +733,6 @@ smart_read (SmartPlugin *smart)
 {
   XfceRc      *rc;
   gchar       *file;
-  const gchar *value;
   
   /* get the plugin config file location */
   file = xfce_panel_plugin_save_location (smart->plugin, TRUE);
@@ -789,6 +805,15 @@ smart_button_release_cb(GtkWidget *w, GdkEventButton *evt,
       case 2:  /* middle */
         if (smart->in_progress)
         {
+           GTimeVal now;
+
+           g_get_current_time(&now);
+           if (now.tv_sec - smart->in_progress_since.tv_sec > PROGRESS_TIME)
+           {
+             smart->in_progress = FALSE;
+             smart_check_upgrades(smart);
+           }
+           else
            xfce_message_dialog(NULL, _("Upgrade In Progress"), GTK_STOCK_DIALOG_WARNING,
                                _("An upgrade is already in progress, please wait."), NULL,
                                GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
@@ -1304,7 +1329,7 @@ smart_about (XfcePanelPlugin *plugin)
   GdkPixbuf *icon;
 
   info = xfce_about_info_new(
-              _("Smart Package Manager"),
+              _("Smart Package Manager - panel plugin"),
               VERSION,
               _("Check for updates using the Smart Package Manager"),
               XFCE_COPYRIGHT_TEXT("2007", "Anders F Bjorklund"),
